/* sync.cc: Synchronization functions for cygwin.

   Copyright 1999 Cygnus Solutions.

   Written by Christopher Faylor <cgf@cygnus.com>

This file is part of Cygwin.

This software is a copyrighted work licensed under the terms of the
Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
details. */

#include <stdlib.h>
#include <time.h>
#include <sys/wait.h>
#include <errno.h>
#include <stdlib.h>
#include "winsup.h"
#include "sync.h"

muto::muto(int inh, const char *name) : sync (-1), visits(0), hthread (0), tid (0)
{
  if (!(bruteforce = CreateSemaphore (inh ? &sec_all_nih : &sec_none_nih, 0, MAXLONG, NULL)))
    {
      DWORD oerr = GetLastError ();
      SetLastError (oerr);
      return;
    }
}

muto::~muto ()
{
  if (bruteforce)
    CloseHandle (bruteforce);
  if (hthread)
    CloseHandle (hthread);
}
  
int
muto::acquire (DWORD ms)
{
  DWORD this_tid = GetCurrentThreadId ();

  if (tid != this_tid)
    while (InterlockedIncrement (&sync) != 0)
      {
	if (ms == 0)
	  {
	    InterlockedDecrement (&sync);
	    SetLastError (ERROR_SEM_TIMEOUT);
	    return 0;
	  }

	switch (WaitForSingleObject (bruteforce, ms))
	  {
	  case WAIT_OBJECT_0:
	    break;
	  case WAIT_TIMEOUT:
	    SetLastError (ERROR_SEM_TIMEOUT);
	    return 0;
	  default:
	    return 0;
	  }

	ms = 0;
      }

  tid = this_tid;
  visits++;
  return 1;
} 

int
muto::release ()
{
  DWORD osync;
  if (tid != GetCurrentThreadId ())
    {
      SetLastError (ERROR_NOT_OWNER);
      return 0;
    }

  if (--visits > 0)
    return 1;
  tid = 0;
  /* FIXME: Do we really need the InterlockedExchange here? */
  if ((osync = InterlockedExchange (&sync, -1)) != 0)
    (void) ReleaseSemaphore (bruteforce, osync, NULL);
  return 1;
}
